import json
import urllib2

from urlresolver import display_settings, relevant_resolvers, common
from urlresolver.plugins.realdebrid import RealDebridResolver
from urlresolver.resolver import ResolverError

from xbmcswift2 import Plugin
from xbmcswift2 import xbmcgui

logger = common.log_utils.Logger.get_logger('realdebrid')
logger.disable()


class CustomRealDebridResolver(RealDebridResolver):

    CLIENT_ID = 'C45DLA7H4QEQ4'
    name = 'Real-Debrid (Csutom)'

    def authorize_resolver(self):
        url = (
            'https://api.real-debrid.com/oauth/v2/device/code'
            '?client_id=%s&new_credentials=yes'
        ) % (self.CLIENT_ID)
        result = self.net.http_GET(url, headers=self.headers).content
        js_result = json.loads(result)
        title = 'URL Resolver Real Debrid Authorization'
        line1 = 'Go to URL: %s' % (js_result['verification_url'])
        line2 = 'When prompted enter: %s' % (js_result['user_code'])
        with common.kodi.CountdownDialog(
                title, line1, line2, countdown=120,
                interval=js_result['interval']) as cd:
            result = cd.start(self.__check_auth, [js_result['device_code']])

        # cancelled
        if result is None: return
        return self._RealDebridResolver__get_token(
            result['client_id'],
            result['client_secret'],
            js_result['device_code'],
        )

    def __check_auth(self, device_code):
        try:
            url = (
                'https://api.real-debrid.com/oauth/v2/device/credentials'
                '?client_id=%s&code=%s'
            ) % (self.CLIENT_ID, device_code)
            content = self.net.http_GET(url, headers=self.headers).content
            js_result = json.loads(content)
        except Exception as e:
            logger.log_debug('Exception during RD auth: %s' % (e))
        else:
            return js_result

    def get_downloads(self, retry=False):
        try:
            url = 'https://api.real-debrid.com/rest/1.0/downloads'
            headers = self.headers
            headers['Authorization'] = 'Bearer %s' % (self.get_setting('token'))
            result = self.net.http_GET(url, headers=headers).content
        except urllib2.HTTPError as e:
            if e.code == 401:
                if retry:
                    raise ResolverError('Real Debrid Auth Failed')
                if self.get_setting('refresh'):
                    self.refresh_token()
                    return self.get_downloads(retry=True)
                else:
                    self.reset_authorization()
                    raise ResolverError('Real Debrid Auth Failed & No Refresh Token')
            raise ResolverError('Real Debrid API Failed: %s' % str(e))
        except Exception as e:
            raise ResolverError('Unexpected Exception during RD Unrestrict: %s' % (e))
        else:
            js_result = json.loads(result)
            return js_result

    def get_account(self, retry=False):
        try:
            url = 'https://api.real-debrid.com/rest/1.0/user'
            headers = self.headers
            headers['Authorization'] = 'Bearer %s' % (self.get_setting('token'))
            result = self.net.http_GET(url, headers=headers).content
        except urllib2.HTTPError as e:
            if e.code == 401:
                if retry:
                    raise ResolverError('Real Debrid Auth Failed')
                if self.get_setting('refresh'):
                    self.refresh_token()
                    return self.get_account(retry=True)
                else:
                    self.reset_authorization()
                    raise ResolverError('Real Debrid Auth Failed & No Refresh Token')
            raise ResolverError('Real Debrid API Failed: %s' % str(e))
        except Exception as e:
            raise ResolverError('Unexpected Exception during RD Unrestrict: %s' % (e))
        else:
            js_result = json.loads(result)
            return js_result


def index():
    downloads = {
        'label': 'My downloads',
        'path': 'plugin://plugin.video.realdebrid/downloads/',
        'is_playable': False,
    }
    account = {
        'label': 'Account',
        'path': 'plugin://plugin.video.realdebrid/account/',
        'is_playable': False,
    }
    ad = {
        'label': 'WTYCZKA POBRANA Z WWW.KODIWPIGULCE.PL',
        'path': 'plugin://plugin.video.realdebrid/',
        'is_playable': False,
    }
    return [downloads, account, ad]


def downloads():
    read_debrid_resolver = get_resolver()
    read_debrid_resolver.login()
 
    downloads = read_debrid_resolver.get_downloads()
    for download in downloads:
        if download['streamable']:
            # use https connection
            download['download'] = download['download'].replace("http://","https://")
            
            info = {}
            if download['filesize']:
                info['size'] = download['filesize']

            yield {
                'label': download['filename'],
                'path': download['download'],
                'is_playable': True,
                'info': info,
            }


def account():
    read_debrid_resolver = get_resolver()
    read_debrid_resolver.login()

    account = read_debrid_resolver.get_account()
    logger.log_debug('Account data: %s' % (account))

    user = account['username']
    id = str(account['id'])
    email = account['email']
    type = account['type'].capitalize()
    points = str(account['points'])

    expiration = account['expiration']
    msec_idx = expiration.find('.')
    if msec_idx >= 0: expiration = expiration[:msec_idx]
    expiration_date = expiration.strip().lower().replace('t', ' ')

    titles = ["ID", "Username", "E-mail", "Type", "Points", "Expiration date"]
    items = [id, user, email, type, points, expiration_date]
    items_formatted = map(option_formatter, titles, items)
    xbmcgui.Dialog().select("Account", items_formatted)


def option_formatter(title, value):
    return '[B]{0}[/B]: {1}'.format(title, value)


def get_resolver(*args, **kwargs):
    return CustomRealDebridResolver(*args, **kwargs)


plugin = Plugin()
plugin.add_url_rule("/", index, name=index.__name__)
plugin.add_url_rule('/downloads/', downloads, name=downloads.__name__)
plugin.add_url_rule('/account/', account, name=account.__name__)
plugin.add_url_rule('/settings/', display_settings, name='settings')

if __name__ == '__main__':
    plugin.run()
